/***************************************************************************
 *                                                                         *
 *  Copyright 1995 Symbios Logic Incorporated.  All rights reserved.       *
 *                                                                         *
 * This software was written by Symbios Logic Inc. to develop and test new *
 * products.  Symbios Logic assumes no liability for its use. This software*
 * is released to the public domain to illustrate certain programming      *
 * techniques for testing the 53c8xx chips                                 *
 *                                                                         *
 ***************************************************************************/

/*           Name:  8XX_TOOL.C
 *          Title:  Validation test code 53C8XX device tools
 *      $Workfile:  $
 *      $Revision:  $
 *       $Modtime:  $
 *     Programmer:  Symbios Logic
 *  Creation Date:  11/29/95
 *
 *  Version History
 *  ---------------
 *
 *    Date    Who?  Description
 *  --------  ----  -------------------------------------------------------
#BeginRevision
 *  03/06/96  SYM   - Made general formatting changes and documentation
 *						  additions
 *						  - Added function C8XX_GetChipType to provide feature
 *						  information on specific devices
 *						  - Added function C8XX_SetBurstSize to set the burst
 *						  size for the device.
 *						  - Fixed problem in init8xx in which the clock divider
 *						  was not being programmed correctly for the 860
 *						  - Changed iowriteDSP to a macro and remove from this file
 *						  - In align_script changed the explicit buffer alignment
 *						  to a call to ByteAlignBuffer.
 *						  - Added script alignment check to CopytoScRam function
 *						  - Removed unneeded else statement in all ScRam functions
 *						  - Added code to enable Clock Doubler in init8xx
 *
#EndRevision
 *
 *  11/29/95  SYM   Initial Revision.
 *
#BeginDescription
 *
 *  This file contains functions which aid in manipulating the 53C8XX
 *  PCI to SCSI devices.  It does not contain an SCSI protocol implemention
 *  functions.
 *
#EndDescription
 *
 *-------------------------------------------------------------------------
 *
 *  $Header: $
 *
 */

#include "valtypes.h"
/* the following two #includes must go in this order */
#include "pci.h"
#include "8xx_pci.h"
#include "8xx_tool.h"
#include "gen_tool.h"
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <dos.h>
#include <mem.h>

/* local global variables */
static VCHAR string[50];

/* function prototypes */
VINT init_8XX(pcidev_record *PCIDevice);
VULONG *align_script(VULONG *original_script, VUINT size);
VINT PollISTAT(VULONG IO_Base, VULONG *vector, VUBYTE *dstat,
					VUBYTE *istat, VUINT *sist, VUINT timeout);
VINT CopyToScRam (pcidev_record *PCIDevice, VULONG far *script, VUINT size);
VINT WriteScRam(pcidev_record *PCIDevice, VUBYTE byte, VINT offset);
VINT ReadScRam(pcidev_record *PCIDevice, VUBYTE *byte, VINT offset);
VINT WriteLongScRam(pcidev_record *PCIDevice, VULONG dword, VINT offset);
VINT ReadLongScRam(pcidev_record *PCIDevice, VULONG *dword, VINT offset);
VINT GenRandSG(VUBYTE far *buf, VUINT size, ti_entry *sg_list, VINT num_entries);
VINT C8XX_GetChipType (pcidev_record *PCIDevice, c8xxdev_record *C8XXDevice);
void C8XX_SetBurstSize(pcidev_record *PCIDevice, VINT burst);


/******************************************************************************
 Function: init_8XX

	Purpose: To initialize a 53C8xx device to a known state
	Input: A pointer to the pcidev_record structure.
		  The base_addr2 and device_id fields of this structure
		  are used in this routine.
	Output: Nothing
	Assumptions: SCSI Scripts are currently not executing
	Restrictions:  None
	Other functions called: IOWrite8 to write initialization values to
				chip registers
				RMWon to turn on single bits in registers
				WriteLog to output exactly what is being
				initialized to a log file
				sprintf to format a string for WriteLog
				delay to hold the SCSI bus reset assertion

******************************************************************************/

VINT init_8XX(pcidev_record *PCIDevice)
{
	VULONG IO_Base;
	c8xxdev_record C8XXDevice;

	if(C8XX_GetChipType(PCIDevice, &C8XXDevice) != C8XX_GOOD)
	{
		WriteLog(LOG_FAIL, LOG_DETAIL, "Unknown device type..Can't initialize\n", NULL);
		return(C8XX_FAIL);
	}

	IO_Base = PCIDevice->base_addr2;

	sprintf(string, "\nInitializing %s...\n", C8XX_DevID2Str(PCIDevice->device_id,
																	  PCIDevice->rev_id));
	WriteLog(LOG_PASS, LOG_DETAIL, string, NULL);

	IOWrite8(IO_Base+ISTAT, 0x40);  /* now reset the chip */
	delay(3);                       /* wait a while, not absolutely necessary */
	IOWrite8(IO_Base+ISTAT, 0x00);  /* clear the reset */

	IOWrite8(IO_Base+SCNTL1, 0x08); /* now reset the SCSI bus */
	delay(3);                       /* wait a while */
	IOWrite8(IO_Base+SCNTL1, 0x00); /* clear the SCSI reset */

	/* clear the interrupt from the SCSI reset, quickest to read the */
	/* entire dword */
	IORead32(IO_Base+SIEN0);

	/* Set up for 16 burst */
	RMWon(IO_Base+DMODE, 0xC0);

	/* Check for a clock doubler */
	if (C8XXDevice.features & C8XX_CLK_DBLR)
	{
		/* turn on the clock doubler */
		/* SET Doubler enable */
		RMWon(IO_Base+STEST1, 0x08);
		/* wait */
		delay(1);
		/* Halt SCSI Clock */
		RMWon(IO_Base+STEST3, 0x20);
		/* Set clock divider */
		RMWoff(IO_Base+SCNTL3, 0xFF);
		RMWon(IO_Base+SCNTL3, 0x05);
		/* Set Clock Doubler Select */
		RMWon(IO_Base+STEST1, 0x04);
		/* Clear Halt SCSI Clock */
		RMWoff(IO_Base+STEST3, 0x20);
	}

	/* Set up Clock divider */
	if (C8XXDevice.features & C8XX_FAST20)
		RMWon(IO_Base+SCNTL3, 0x05);
	else
		RMWon(IO_Base+SCNTL3, 0x03);

	/* Set up chip ID and Response to reselection */
	RMWon(IO_Base+SCID, 0x47);

	/* Set up Response ID */
	RMWon(IO_Base+RESPID0, 0x80);

	/* Enabling SCSI parity checking and Assert ATN on Parity Error */
	RMWon(IO_Base+SCNTL0, 0x0A);

	/* Setting up selection timeout to 250ms */
	RMWon(IO_Base+STIME0, 0x0C);

	/* look for differential board by looking at the state of GPIO3 */
	if (!(IORead8(IO_Base+GPREG) & 0x08))
	{
		/* Differential board detected setting up diff mode */
		RMWon(IO_Base+STEST2, 0x20);
	}

	return(C8XX_GOOD);

}


/******************************************************************************
 Function: align_script
	Purpose: To align a SCSI Script to a dword boundary
	Input: A pointer to the original script array           
	       The size of the script array
	Output: A pointer to an aligned script array.  If the original
		array is already aligned then no memory is allocated.
		If not, new memory is allocated and aligned and the
		script is copied into this aligned memory and a pointer
		to this array is returned. If memory cannot be allocated
		an error is printed and the program will exit.
	Assumptions: none
	Restrictions:  none
	Other functions called: FP_OFF to get offset of script array
				malloc to allocate a new array if necessary
				_fmemcpy to copy script array
				printf to print error message if memory
				cannot be allocated

******************************************************************************/

VULONG far *align_script(VULONG *original_script, VUINT size)
{
	VUBYTE far *script_storage;
	VULONG far *script;

	 /* Make sure script starts on a 32-bit boundry */
	if (FP_OFF(original_script) & 0x03)
	{
		/* If the script array is not on a dword boundary then we need */
		/* to malloc a new array and dword align it */
		/* Not very efficient but it works */
		script_storage = (VUBYTE far *) malloc(size + 3);

		if (script_storage == NULL)
		{
			WriteLog(LOG_FAIL, LOG_DETAIL, "\nError: Unable to allocate enough "
									"memory to align script!\n", NULL);
			exit(1);
		}

		/* Align the new array */
		script = (VULONG far *) ByteAlignBuffer(script_storage, 0);

		/* Copy the script into the new array */
		_fmemcpy(script, original_script, size);

	}
	else
		script = original_script;

	return(script);

}

/******************************************************************************
 Function: PollISTAT
	Purpose: Poll the ISTAT register until an interrupt is detected or
		 a timeout occurs.  This function will clear all interrupts
		 including stacked ones.
	Input: The IO Base address of the 53C8XX part
			 Pointers to variables to hold the values of dstat, istat, sist
			 and an interrupt vector
			 A timeout period in seconds, a timeout of 0 equals no timeout.
	Output: If a timeout occurs and no interrupt is detected then the
		function returns C8XX_POLLTO.
		If an interrupt is detected then the dstat, istat, sist and
		if necessary vector variables are filled in and the function
		returns C8XX_GOOD.
	Assumptions:   none
	Restrictions:  none
	Other functions called: IORead8 to read the ISTAT register and
				dstat register
				IORead32 to read the vector regiser and
				the sist register
				Get_Time to get the current time
				HasTimeElapsed to determine if the timeout
				has expired

******************************************************************************/

VINT PollISTAT(VULONG IO_Base, VULONG *vector, VUBYTE *dstat,
					VUBYTE *istat, VUINT *sist, VUINT timeout)

{
	VULONG StartTime;               /* Used to calc timeout */

	/* no timeout specified, loop forever */
	if (!timeout)
		while (!(IORead8(IO_Base+ISTAT) & 0x3));

	else /* a timeout is specified */
	{
		/* get the current time and poll for that long */
		StartTime = Get_Time();
		while (!(IORead8(IO_Base+ISTAT) & 0x3))
			if (HasTimeElapsed(StartTime, timeout)) return(C8XX_POLLTO);
	}

	/* an interrupt was recieved, initialize the variable and fill them in */
	*istat = 0;
	*dstat = 0;
	*sist = 0;
	*vector = 0xFFFFFFFFl;

	/* get all interrupts, including stacked ones */
	while (IORead8(IO_Base+ISTAT) & 0x03)
	{
		*istat |= IORead8(IO_Base+ISTAT);

		if (*istat & 0x1)                     /* DMA Interrupt */
			*dstat |= IORead8(IO_Base+DSTAT);

		if (*istat & 0x2)                     /* SCSI Interrupt */
			*sist |= (VUINT) (IORead32(IO_Base+SIEN0) >> 16);
	}

	/* a script interrupt instruction caused the interrupt */
	/* so get the vector */
	if (*dstat & 0x4)
		*vector = IORead32(IO_Base+DSPS);

	/* this is in case we didn't get a DMA and need the state of the */
	/* DMA fifo full bit */
	*dstat |= IORead8(IO_Base+DSTAT);

	return(C8XX_GOOD);

}

/******************************************************************************
 Function: CopyToScRam
	Purpose: Copy a script to script ram.  This function uses a memory
		 to memory move to copy the script to script ram
	Input: A pointer to the pcidev_record data structure
		  The base_addr2 and 3 fields are used in this structure
			 A pointer to the script to be copied
			 The size of the script being copied
	Output: C8XX_GOOD upon successful completion of the copy
		C8XX_RAMCOPYFAILURE upon failure of the copy
	Assumptions: SCSI Scripts are currently not executing
			  The device being accessed currently has scripts ram
			  available.
	Restrictions:  Do not call this function while scripts are executing
	Other functions called: Bytealignbuffer to align the mem to mem
				move script
				iowriteDSP to start the mem to mem script
				pollISTAT to wait for an interrupt for
				completion of mem to mem move

******************************************************************************/

VINT CopyToScRam (pcidev_record *PCIDevice, VULONG far *script, VUINT size)

{
	VUBYTE m2mscript[24];           /* array that holds mem to mem move */
	VULONG *m2mscript_ptr;          /* alignment pointer into array above */
	VINT poll_ret;             /* return value from pollISTAT */
	VUBYTE istat, dstat;       /* Interrupt regs filled in by PollISTAT */
	VUINT sist;
	VULONG vector;

	/* First check for script alignment */
	if (getPhysAddr(script) & 0x3) return(C8XX_RAMCOPYFAILURE);

	/* Since scripts must be dword aligned we need to */
	/* Find a dword aligned spot in the array */
	m2mscript_ptr = ByteAlignBuffer(m2mscript, 0);

	/* set up mem to mem move instruction for copy */
	m2mscript_ptr[0] = 0xC0000000l | size;      /* Mem move opcode */
	m2mscript_ptr[1] = getPhysAddr(script);     /* Source address */
	m2mscript_ptr[2] = PCIDevice->base_addr3;   /* Destination Address */
	m2mscript_ptr[3] = 0x98080000l;             /* Int instruction */
	m2mscript_ptr[4] = 0xCCCCCCCCl;             /* Int vector */

	/* start copy */
	iowriteDSP(PCIDevice->base_addr2, getPhysAddr(m2mscript_ptr));

	poll_ret = PollISTAT(PCIDevice->base_addr2, &vector, &dstat, &istat, &sist, 5);

	if (poll_ret == C8XX_GOOD)   /* got an interrupt */
	{
		/* got the correct interrupt */
		if ((dstat & C8XX_INT_SIR) && (vector & C8XX_COPYCOMPLETE))
			return(C8XX_GOOD);

		else return(C8XX_RAMCOPYFAILURE);  /* didn't get the correct interrupt */
	}

	/* any other return failure  */
	else return(C8XX_RAMCOPYFAILURE);

}

/******************************************************************************
 Function: WriteScRam
	Purpose: write a byte to script ram.  This function uses a memory
		 to memory move to write to script ram
	Input: A pointer to the pcidev_record data structure           
		  The base_addr2 and 3 fields are used in this structure
	       The value of the byte to be written
	       The offset in scripts ram where the byte is to be written
	Output: C8XX_GOOD upon successful completion of the write
		C8XX_RAMWRITEFAILURE upon failure of the write
	Assumptions: SCSI Scripts are currently not executing
		     The device being accessed currently has scripts ram
	Restrictions:  Do not call this function while scripts are executing
	Other functions called: Bytealignbuffer to align the mem to mem
				move script
				iowriteDSP to start the mem to mem script
				pollISTAT to wait for an interrupt for
				completion of mem to mem move
									  
******************************************************************************/

VINT WriteScRam(pcidev_record *PCIDevice, VUBYTE byte, VINT offset)

{

	VUBYTE m2mscript[24];           /* array that holds mem to mem move */
	VULONG *m2mscript_ptr;          /* alignment pointer into array above */
	VINT poll_ret;             /* return value from pollISTAT */
	VUBYTE istat, dstat;       /* Interrupt regs filled in by PollISTAT */
	VUINT sist;
	VULONG vector;
	VUBYTE ram_dword[8];                    /* Holds the value to be written */
	VULONG *ram_dword_ptr;     /* alignment pointer into array above */

	/* Since scripts must be dword aligned we need to */
	/* Find a dword aligned spot in the array */
	m2mscript_ptr = ByteAlignBuffer(m2mscript, 0);
	/* align the buffer that will hold the data to be written */
	ram_dword_ptr = ByteAlignBuffer(ram_dword, 0);

	/* align the byte to the proper lane for writing to script ram */
	*ram_dword_ptr = 0;
	*ram_dword_ptr |= (VULONG) byte << ((offset&0x3)*8);

	/* set up mem to mem move instruction for copy */
	m2mscript_ptr[0] = 0xC0000001l;
	/* We need to set the source address exactly to the byte we want to move */
	/* This is done by adding the lower 2 bits of the offset to the address */
	/* as this is where the byte we need to move actually is */
	m2mscript_ptr[1] = getPhysAddr(ram_dword_ptr) + (offset&0x3);
	m2mscript_ptr[2] = PCIDevice->base_addr3+offset;
	m2mscript_ptr[3] = 0x98080000l;
	m2mscript_ptr[4] = 0xCCCCCCCCl;

	/* start read */
	iowriteDSP(PCIDevice->base_addr2, getPhysAddr(m2mscript_ptr));

	poll_ret = PollISTAT(PCIDevice->base_addr2, &vector, &dstat, &istat, &sist, 2);

	if (poll_ret == C8XX_GOOD)   /* got an interrupt */
	{
		/* got the correct interrupt */
		if ((dstat & C8XX_INT_SIR) && (vector & C8XX_COPYCOMPLETE))
			return(C8XX_GOOD);

		else return(C8XX_RAMWRITEFAILURE); /* didn't get the correct interrupt */
	}

	/* Any other return value fails */
	else return(C8XX_RAMWRITEFAILURE);

}

/******************************************************************************
 Function: ReadScRam
	Purpose: read a byte from script ram.  This function uses a memory
		 to memory move to read from script ram
	Input: A pointer to the pcidev_record data structure           
		  The base_addr2 and 3 fields are used in this structure
	       A pointer to the variable to recieve the byte
	       The offset of the byte to be read
	Output: C8XX_GOOD upon successful completion of the read
		C8XX_RAMREADFAILURE upon failure of the read
	Assumptions: SCSI Scripts are currently not executing
		     The device being accessed currently has scripts ram
	Restrictions:  Do not call this function while scripts are executing
	Other functions called: Bytealignbuffer to align the mem to mem
				move script
				iowriteDSP to start the mem to mem script
				pollISTAT to wait for an interrupt for
				completion of mem to mem move
									  
******************************************************************************/

VINT ReadScRam(pcidev_record *PCIDevice, VUBYTE *byte, VINT offset)

{

	VUBYTE m2mscript[24];           /* array that holds mem to mem move */
	VULONG *m2mscript_ptr;          /* alignment pointer into array above */
	VINT poll_ret;             /* return value from pollISTAT */
	VUBYTE istat, dstat;       /* Interrupt regs filled in by PollISTAT */
	VUINT sist;
	VULONG vector;
	VUBYTE ram_dword[8];                    /* Holds the value to be written */
	VULONG *ram_dword_ptr;     /* alignment pointer into array above */

	/* align the mem 2 mem script */
	m2mscript_ptr = ByteAlignBuffer(m2mscript, 0);
	/* align the buffer that will hold the data to be read */
	ram_dword_ptr = ByteAlignBuffer(ram_dword, 0);

	/* set up mem to mem move instruction for copy */
	m2mscript_ptr[0] = 0xC0000004l;;
	/* The offset is automatically aligned for the move */
	m2mscript_ptr[1] = (PCIDevice->base_addr3+offset) & 0xFFFFFFFCl;
	m2mscript_ptr[2] = getPhysAddr(ram_dword_ptr);
	m2mscript_ptr[3] = 0x98080000l;
	m2mscript_ptr[4] = 0xCCCCCCCCl;

	/* start read */
	iowriteDSP(PCIDevice->base_addr2, getPhysAddr(m2mscript_ptr));

	poll_ret = PollISTAT(PCIDevice->base_addr2, &vector, &dstat, &istat, &sist, 2);

	if (poll_ret == C8XX_GOOD)  /* got an interrupt */
	{
		/* got the correct interrupt */
		if ((dstat & C8XX_INT_SIR) && (vector & C8XX_COPYCOMPLETE))
		{
			/* place the byte in the variable */
			*byte = (VUBYTE) ((*ram_dword_ptr >> ((offset&0x3)*8)) & 0xFF);
			return(C8XX_GOOD);
		}
		else return(C8XX_RAMREADFAILURE);  /* didn't get the correct interrupt */
	}

	/* Any other return value fails */
	else return(C8XX_RAMWRITEFAILURE);

}

/******************************************************************************
 Function: WriteLongScRam
	Purpose: write a dword to script ram.  This function uses a memory
		 to memory move to write to script ram
	Input: A pointer to the pcidev_record data structure           
		  The base_addr2 and 3 fields are used in this structure
	       The value of the dword to be written
	       The offset in scripts ram where the dword is to be written
	Output: C8XX_GOOD upon successful completion of the write
		C8XX_RAMWRITEFAILURE upon failure of the write
	Assumptions: SCSI Scripts are currently not executing
		     The device being accessed currently has scripts ram
	Restrictions:  Do not call this function while scripts are executing
	Other functions called: Bytealignbuffer to align the mem to mem
				move script
				iowriteDSP to start the mem to mem script
				pollISTAT to wait for an interrupt for
				completion of mem to mem move
									  
******************************************************************************/

VINT WriteLongScRam(pcidev_record *PCIDevice, VULONG dword, VINT offset)

{

	VUBYTE m2mscript[24];           /* array that holds mem to mem move */
	VULONG *m2mscript_ptr;          /* alignment pointer into array above */
	VINT poll_ret;             /* return value from pollISTAT */
	VUBYTE istat, dstat;       /* Interrupt regs filled in by PollISTAT */
	VUINT sist;
	VULONG vector;
	VUBYTE ram_dword[8];                    /* Holds the value to be written */
	VULONG *ram_dword_ptr;     /* alignment pointer into array above */

	/* align the mem 2 mem script */
	m2mscript_ptr = ByteAlignBuffer(m2mscript, 0);
	/* align the buffer that will hold the data to be written */
	ram_dword_ptr = ByteAlignBuffer(ram_dword, 0);

	/* align the dword to be written */
	*ram_dword_ptr = dword;

	/* set up mem to mem move instruction for copy */
	m2mscript_ptr[0] = 0xC0000004l;;
	m2mscript_ptr[1] = getPhysAddr(ram_dword_ptr) + (offset&0x3);
	m2mscript_ptr[2] = (PCIDevice->base_addr3+offset) & 0xFFFFFFFCl;
	m2mscript_ptr[3] = 0x98080000l;
	m2mscript_ptr[4] = 0xCCCCCCCCl;

	/* start read */
	iowriteDSP(PCIDevice->base_addr2, getPhysAddr(m2mscript_ptr));

	poll_ret = PollISTAT(PCIDevice->base_addr2, &vector, &dstat, &istat, &sist, 2);

	if (poll_ret == C8XX_GOOD)   /* got an interrupt */
	{
		/* got the correct interrupt */
		if ((dstat & C8XX_INT_SIR) && (vector & C8XX_COPYCOMPLETE))
			return(C8XX_GOOD);

		else return(C8XX_RAMWRITEFAILURE);      /* didn't get the correct interrupt */
	}

	/* Any other return value fails */
	else return(C8XX_RAMWRITEFAILURE);

}

/******************************************************************************
 Function: ReadLongScRam
	Purpose: read a dword from script ram.  This function uses a memory
		 to memory move to read script ram
	Input: A pointer to the pcidev_record data structure
		  The base_addr2 and 3 fields are used in this structure
	       A pointer to the variable to recieve the dword
	       The offset of the dword to be read
	Output: C8XX_GOOD upon successful completion of the read
		C8XX_RAMREADFAILURE upon failure of the read
	Assumptions: SCSI Scripts are currently not executing
		     The device being accessed currently has scripts ram
	Restrictions:  Do not call this function while scripts are executing
	Other functions called: Bytealignbuffer to align the mem to mem
				move script
				iowriteDSP to start the mem to mem script
				pollISTAT to wait for an interrupt for
				completion of mem to mem move

******************************************************************************/
VINT ReadLongScRam(pcidev_record *PCIDevice, VULONG *dword, VINT offset)

{

	VUBYTE m2mscript[24];           /* array that holds mem to mem move */
	VULONG *m2mscript_ptr;          /* alignment pointer into array above */
	VINT poll_ret;             /* return value from pollISTAT */
	VUBYTE istat, dstat;       /* Interrupt regs filled in by PollISTAT */
	VUINT sist;
	VULONG vector;
	VUBYTE ram_dword[8];                    /* Holds the value to be written */
	VULONG *ram_dword_ptr;     /* alignment pointer into array above */


	/* align the mem 2 mem script */
	m2mscript_ptr = ByteAlignBuffer(m2mscript, 0);
	/* align the buffer that will hold the data to be written */
	ram_dword_ptr = ByteAlignBuffer(ram_dword, 0);

	/* set up mem to mem move instruction for copy */
	m2mscript_ptr[0] = 0xC0000004l;;
	m2mscript_ptr[1] = (PCIDevice->base_addr3+offset) & 0xFFFFFFFCl;
	m2mscript_ptr[2] = getPhysAddr(ram_dword_ptr);
	m2mscript_ptr[3] = 0x98080000l;
	m2mscript_ptr[4] = 0xCCCCCCCCl;

	/* start read */
	iowriteDSP(PCIDevice->base_addr2, getPhysAddr(m2mscript_ptr));

	poll_ret = PollISTAT(PCIDevice->base_addr2, &vector, &dstat, &istat, &sist, 2);

	if (poll_ret == C8XX_GOOD)   /* got an interrupt */
	{
		/* got the correct interrupt */
		if ((dstat & C8XX_INT_SIR) && (vector & C8XX_COPYCOMPLETE))
		{
			*dword = *ram_dword_ptr;
			return(C8XX_GOOD);
		}
		else return(C8XX_RAMREADFAILURE); /* didn't get the correct interrupt */
	}

	/* Any other return value fails */
	else return(C8XX_RAMWRITEFAILURE);

}

/******************************************************************************
 Function: GenRandSG
	Purpose: Generate a random scatter gather list.  The list is generated
		 in scripts table indirect format.  The function generates
		 random entries by dividing the buffer up into equal slices
		 and then generating a random number within each slice.  The
		 byte count for entry x then is equal to to the distance
		 to the slice boundary for entry x-1 plus some random number
		 between 0 and the slice.  This allows for relatively equal
		 distribution of scatter gather entries while maintaining a
		 relatively high amount of randomness.
	Input: A pointer to the buffer for which the list is to be generated
			 The size of the buffer
			 A pointer to a scatter gather table (table indirect entries)
			 The number of scatter gather entries desired
	Output: This function always returns C8XX_GOOD
	Assumptions: The number of entries is <= the size of the buffer
			  The function randomize was called prior to calling this
			  function
	Restrictions: The number of entries must be <= the size of the buffer
				Only call randomize once during the execution of the code
				(i.e. do not repeatedly call randomize as this will
				 cause not random numbers to be generated)
	Other functions called: getPhysAddr to get the physical address of the
				buffer
				rand to generate random numbers

******************************************************************************/

VINT GenRandSG(VUBYTE far *buf, VUINT size, ti_entry *sg_list, VINT num_entries)

{

	VUINT slice;                            /* An equal slice of the buffer base on num_entries */
	VULONG to_slice_boundry;        /* Distance to next slice boundary */
	VINT i;                                                 /* counting variable */

	/* num_entries = 1 is a special case so we handle it */
	/* separately */
	if (num_entries == 1)
	{
		sg_list[0].address = getPhysAddr(buf);
		sg_list[0].count = size;
		return(C8XX_GOOD);
	}

	/* divide buffer into num_entries equal pieces */
	slice = size/num_entries;

	/* generate the first entry */
	sg_list[0].address = getPhysAddr(buf);
	/* The + 1 prevents us from getting a count of 0, a bad thing */
	sg_list[0].count = (rand() % slice) + 1;

	for (i=1; i<num_entries-1; i++)
	{
		/* calculate starting address of this entry */
		sg_list[i].address = sg_list[i-1].address + sg_list[i-1].count;
		/* figure out how far it is to the slice boundary */
		to_slice_boundry = (getPhysAddr(buf) + i*slice) - sg_list[i].address;
		/* generate the byte count by adding the count to the slice boundary */
		/* plus a random number between 1 and the slice size */
		/* The +1 prevents us from getting a count of 0, a bad thing */
		sg_list[i].count = to_slice_boundry + (rand() % slice) + 1;
	}

	/* handle the last entry, this is just the remainder of the bytes */
	/* in the buffer to be transferred */
	sg_list[num_entries-1].address = sg_list[num_entries-2].address +
												sg_list[num_entries-2].count;
	sg_list[num_entries-1].count = (getPhysAddr(buf) + size) -
												sg_list[num_entries-1].address;

	return(C8XX_GOOD);

}

/******************************************************************************
 Function: C8XX_GetChipType
	Purpose: Return the device type that is being tested
	Input:  Pointer to PCIDevice data structure and Pointer to C8XXDevice data
			  structure
	Output: Filled in C8XXDevice data structure if the device is recognized.
			  If not, C8XXDevice is unknown and UNKNOWN_DEVICE_TYPE is returned.
	Assumptions: none
	Restrictions: none
	Other functions called: none

******************************************************************************/

VINT C8XX_GetChipType (pcidev_record *PCIDevice, c8xxdev_record *C8XXDevice)

{
	VUINT dev_id;
	VUBYTE rev_id;

	/* Local variables are more efficient to use here */
	dev_id = PCIDevice->device_id;
	rev_id = PCIDevice->rev_id;

	switch (dev_id)
	{
		case C810_DEVICE_ID:
		{
			/* Could be 810 or 810a, check revID register to tell */
			if ((rev_id & 0xF0) == C810_VERSION_ID)
			{
				C8XXDevice->device = C8XX_53C810;
				C8XXDevice->features = C8XX_53C810_FEAT;
				C8XXDevice->fifo_size_a = C8XX_53C810_FIFO_A;
				C8XXDevice->fifo_size_b = C8XX_53C810_FIFO_B;
				C8XXDevice->max_burst_a = C8XX_53C810_BRST_A;
				C8XXDevice->max_burst_b = C8XX_53C810_BRST_B;
				C8XXDevice->max_offset = C8XX_53C810_OFFSET;
				return(C8XX_GOOD);
			}
			if ((rev_id & 0xF0) == C810A_VERSION_ID)
			{
				C8XXDevice->device = C8XX_53C810A;
				C8XXDevice->features = C8XX_53C810A_FEAT;
				C8XXDevice->fifo_size_a = C8XX_53C810A_FIFO_A;
				C8XXDevice->fifo_size_b = C8XX_53C810A_FIFO_B;
				C8XXDevice->max_burst_a = C8XX_53C810A_BRST_A;
				C8XXDevice->max_burst_b = C8XX_53C810A_BRST_B;
				C8XXDevice->max_offset = C8XX_53C810A_OFFSET;
				return(C8XX_GOOD);
			}
			if ((rev_id & 0xF0) == C810ALV_VERSION_ID)
			{
				C8XXDevice->device = C8XX_53C810ALV;
				C8XXDevice->features = C8XX_53C810ALV_FEAT;
				C8XXDevice->fifo_size_a = C8XX_53C810ALV_FIFO_A;
				C8XXDevice->fifo_size_b = C8XX_53C810ALV_FIFO_B;
				C8XXDevice->max_burst_a = C8XX_53C810ALV_BRST_A;
				C8XXDevice->max_burst_b = C8XX_53C810ALV_BRST_B;
				C8XXDevice->max_offset = C8XX_53C810ALV_OFFSET;
				return(C8XX_GOOD);
			}
			break;
		}
		case C815_DEVICE_ID:
		{
			if ((rev_id & 0xF0) == C815_VERSION_ID)
			{
				C8XXDevice->device = C8XX_53C815;
				C8XXDevice->features = C8XX_53C815_FEAT;
				C8XXDevice->fifo_size_a = C8XX_53C815_FIFO_A;
				C8XXDevice->fifo_size_b = C8XX_53C815_FIFO_B;
				C8XXDevice->max_burst_a = C8XX_53C815_BRST_A;
				C8XXDevice->max_burst_b = C8XX_53C815_BRST_B;
				C8XXDevice->max_offset = C8XX_53C815_OFFSET;
				return(C8XX_GOOD);
			}
			break;
		}
		case C825_DEVICE_ID:
		{
			/* Could be 825 or 825a, check revID register to tell */
			if ((rev_id & 0xF0) == C825_VERSION_ID)
			{
				C8XXDevice->device = C8XX_53C825;
				C8XXDevice->features = C8XX_53C825_FEAT;
				C8XXDevice->fifo_size_a = C8XX_53C825_FIFO_A;
				C8XXDevice->fifo_size_b = C8XX_53C825_FIFO_B;
				C8XXDevice->max_burst_a = C8XX_53C825_BRST_A;
				C8XXDevice->max_burst_b = C8XX_53C825_BRST_B;
				C8XXDevice->max_offset = C8XX_53C825_OFFSET;
				return(C8XX_GOOD);
			}
			if ((rev_id & 0xF0) == C825A_VERSION_ID)
			{
				C8XXDevice->device = C8XX_53C825A;
				C8XXDevice->features = C8XX_53C825A_FEAT;
				C8XXDevice->fifo_size_a = C8XX_53C825A_FIFO_A;
				C8XXDevice->fifo_size_b = C8XX_53C825A_FIFO_B;
				C8XXDevice->max_burst_a = C8XX_53C825A_BRST_A;
				C8XXDevice->max_burst_b = C8XX_53C825A_BRST_B;
				C8XXDevice->max_offset = C8XX_53C825A_OFFSET;
				return(C8XX_GOOD);
			}
			break;
		}
		case C810AP_DEVICE_ID:
		{
			if ((rev_id & 0xF0) == C810AP_VERSION_ID)
			{
				C8XXDevice->device = C8XX_53C810A;
				C8XXDevice->features = C8XX_53C810A_FEAT;
				C8XXDevice->fifo_size_a = C8XX_53C810A_FIFO_A;
				C8XXDevice->fifo_size_b = C8XX_53C810A_FIFO_B;
				C8XXDevice->max_burst_a = C8XX_53C810A_BRST_A;
				C8XXDevice->max_burst_b = C8XX_53C810A_BRST_B;
				C8XXDevice->max_offset = C8XX_53C810A_OFFSET;
				return(C8XX_GOOD);
			}
			break;
		}
		case C860_DEVICE_ID:
		{
			if ((rev_id & 0xF0) == C860_VERSION_ID)
			{
				C8XXDevice->device = C8XX_53C860;
				C8XXDevice->features = C8XX_53C860_FEAT;
				C8XXDevice->fifo_size_a = C8XX_53C860_FIFO_A;
				C8XXDevice->fifo_size_b = C8XX_53C860_FIFO_B;
				C8XXDevice->max_burst_a = C8XX_53C860_BRST_A;
				C8XXDevice->max_burst_b = C8XX_53C860_BRST_B;
				C8XXDevice->max_offset = C8XX_53C860_OFFSET;
				return(C8XX_GOOD);
			}
			if ((rev_id & 0xF0) == C860LV_VERSION_ID)
			{
				C8XXDevice->device = C8XX_53C860LV;
				C8XXDevice->features = C8XX_53C860LV_FEAT;
				C8XXDevice->fifo_size_a = C8XX_53C860LV_FIFO_A;
				C8XXDevice->fifo_size_b = C8XX_53C860LV_FIFO_B;
				C8XXDevice->max_burst_a = C8XX_53C860LV_BRST_A;
				C8XXDevice->max_burst_b = C8XX_53C860LV_BRST_B;
				C8XXDevice->max_offset = C8XX_53C860LV_OFFSET;
				return(C8XX_GOOD);
			}
			break;
		}
		case C875_DEVICE_ID:
		{
			if ((rev_id & 0xF0) == C875_VERSION_ID)
			{
				C8XXDevice->device = C8XX_53C875;
				C8XXDevice->features = C8XX_53C875_FEAT;
				/* Rev. 2 and above 875's have clock doubler */
				if ((rev_id & 0x0F) >= 2)
					C8XXDevice->features |= C8XX_CLK_DBLR;
				C8XXDevice->fifo_size_a = C8XX_53C875_FIFO_A;
				C8XXDevice->fifo_size_b = C8XX_53C875_FIFO_B;
				C8XXDevice->max_burst_a = C8XX_53C875_BRST_A;
				C8XXDevice->max_burst_b = C8XX_53C875_BRST_B;
				C8XXDevice->max_offset = C8XX_53C875_OFFSET;
				return(C8XX_GOOD);
			}
			break;
		}
		case C885_DEVICE_ID:
		{
			if ((rev_id & 0xF0) == C885_VERSION_ID)
			{
				C8XXDevice->device = C8XX_53C885;
				C8XXDevice->features = C8XX_53C885_FEAT;
				C8XXDevice->fifo_size_a = C8XX_53C885_FIFO_A;
				C8XXDevice->fifo_size_b = C8XX_53C885_FIFO_B;
				C8XXDevice->max_burst_a = C8XX_53C885_BRST_A;
				C8XXDevice->max_burst_b = C8XX_53C885_BRST_B;
				C8XXDevice->max_offset = C8XX_53C885_OFFSET;
				return(C8XX_GOOD);
			}
			break;
		}
		case C895_DEVICE_ID:
		{
			if ((rev_id & 0xF0) == C895_VERSION_ID)
			{
				C8XXDevice->device = C8XX_53C895;
				C8XXDevice->features = C8XX_53C895_FEAT;
				C8XXDevice->fifo_size_a = C8XX_53C895_FIFO_A;
				C8XXDevice->fifo_size_b = C8XX_53C895_FIFO_B;
				C8XXDevice->max_burst_a = C8XX_53C895_BRST_A;
				C8XXDevice->max_burst_b = C8XX_53C895_BRST_B;
				C8XXDevice->max_offset = C8XX_53C895_OFFSET;
				return(C8XX_GOOD);
			}
			break;
		}
		case C896_DEVICE_ID:
		{
			if ((rev_id & 0xF0) == C896_VERSION_ID)
			{
				C8XXDevice->device = C8XX_53C896;
				C8XXDevice->features = C8XX_53C896_FEAT;
				C8XXDevice->fifo_size_a = C8XX_53C896_FIFO_A;
				C8XXDevice->fifo_size_b = C8XX_53C896_FIFO_B;
				C8XXDevice->max_burst_a = C8XX_53C896_BRST_A;
				C8XXDevice->max_burst_b = C8XX_53C896_BRST_B;
				C8XXDevice->max_offset = C8XX_53C896_OFFSET;
				return(C8XX_GOOD);
			}
			break;
		}
	}
	return(C8XX_UNKNOWN_DEVICE);
}

/******************************************************************************
 Function: C8XX_SetBurstSize
	Purpose: Set the burst size of the device being tested
	Input: A pointer to the PCIDevice data structure and the desired burst
			 size.  Burst size macros C8XX_BURST_N should be used for the
			 burst parameter to insure compatibility.
	Output: Sets the burst size for the device pointed to by PCIDevice
	Assumptions: That the burst is valid for this device
	Restrictions: It would not be wise to call this function with a burst
					  size that is invalid for the device.
	Other functions called: RMWoff/on to set and clear bits in regs

******************************************************************************/

void C8XX_SetBurstSize(pcidev_record *PCIDevice, VINT burst)

{

	/* first clear the current burst size */
	RMWoff(PCIDevice->base_addr2+CTEST5, 0x04);
	RMWoff(PCIDevice->base_addr2+DMODE, 0xC0);

	/* Since the burst definitions are linearly incrementing then I */
	/* am just going to move the bits to the proper locations in the */
	/* proper registers.  This code assumes that the burst definitions */
	/* are linearly incrementing */
	/* Since the setting of burst sizes of 32, 64 and 128 is in bit 2 */
	/* of the CTEST5 register I just need to mask off other bits */
	/* and write the same bit location in the variable out to the register */
	RMWon(PCIDevice->base_addr2+CTEST5, (burst&0x4));
	/* The lower two burst bits go in DMODE so we shift the lower 2 bits */
	/* up to the upper part of DMODE where the burst size is set */
	RMWon(PCIDevice->base_addr2+DMODE, ((burst&0x3)<<6));

}
