/***************************************************************************
 *                                                                         *
 *  Copyright 1995 Symbios Logic Incorporated.  All rights reserved.       *
 *                                                                         *
 * This software was written by Symbios Logic Inc. to develop and test new *
 * products.  Symbios Logic assumes no liability for its use. This software*
 * is released to the public domain to illustrate certain programming      *
 * techniques for testing the 53c8xx chips                                 *
 *                                                                         *
 ***************************************************************************/

/*           Name:  VALIDATE.C
 *          Title:  Validation test code main program
 *      $Workfile:  $
 *      $Revision:  $
 *       $Modtime:  $
 *     Programmer: Symbios Logic 
 *  Creation Date:  11/29/95
 *
 *  Version History
 *  ---------------
 *
 *    Date    Who?  Description
 *  --------  ----  -------------------------------------------------------
#BeginRevision
 *  03/06/96  SYM   - Made general formatting changes and documentation
 *						  additions
 *						  - Added call to GetChipType to get device specific
 *						  features for use in main.
 *						  - Changed algorithm for determining whether or not to
 *						  try and get the Script RAM base address to use C8XXDevice
 *						  structure returned by GetChipType instead of explicitly
 *						  looking for devices that have script ram.
 *						  - Changed final PASS/FAIL printout to detail log entries
 *
#EndRevision
 *
 *  11/29/95  SYM   Initial Revision.
 *
#BeginDescription
 *
 * This file contains the function main for starting the validation
 * test code.  It will initialize all devices, call the parsecmdline
 * function and finally call the test function.
 *
#EndDescription
 *
 *-------------------------------------------------------------------------
 *
 *  $Header: $
 *
 */

/* Do not change the ordering of these include files */
#include "valtypes.h"
#include "pci.h"
#include "test.h"
#include "8xx_pci.h"
#include "8xx_tool.h"
#include "8xx_scsi.h"
#include "validate.h"
#include "gen_tool.h"
#include <stdio.h>
#include <conio.h>
#include <stdlib.h>
#include <string.h>

/* Global Variables */
VCHAR *logfilename_ptr;

/* Local Global variables */
static pcidev_record *PCIDevice_ptr[MAX_DEVICES];
static pcidev_record PCIDevice[MAX_DEVICES];
static VCHAR string[60];
static VCHAR logfilename[13];
static cmdlineflags flags;

/******************************************************************************
 Function: main
	Purpose : To initialize devices and standard data structures.  It
		  also calls the parsecmdline function to parse the command
		  line options and lastly calls the test function to execute
		  the specific test implemented.
	Input: Argc and argv which contain the command line parameters
	Output: Nothing
	Assumptions: none
	Restrictions: none
	Other functions called: strncpy to copy the default file name to a
					string
				WriteLog to output various device information
				sprintf to format strings for WriteLog
				fprintf to output things only to the screen
				exit to exit the program on errors
				The following initialization routines are                 
					called in this order:
				parsecmdline to parse command line parameters
				pci_check_pci_bios to ensure the existance of
					a pci bios
				C8XX_BuildDeviceList to build a list of C8XX
					devices in the system
				C8XX_ChoosePCIDevice to allow the user to 
					select a specific pci device to be
					used for testing
				C8XXGetChipType to get chip specific info
				C8XX_GetIOBase to get the IO Base address of
					a 53C8XX device
				C8XX_GetRAMBase to get the Script RAM Base
					Address of a 53C8XX device
				PCI_print_config to output the config registers
					of a specific device.
				init_8XX to initialize a 53C8XX device to a
					known state
				test to execute the programs specific chip test

******************************************************************************/

void main(VINT argc, VCHAR **argv)
{
	VINT NumPCIDevices;             /* Holds number of Symbios PCI devices found */
	VINT SelPCIDevice;              /* Holds index of device selected for testing */
	VINT i;                                         /* counting variable */
	pci_bios_info pcibios;  /* PCI Bios information */
	c8xxdev_record C8XXDevice;      /* Structure for holding chip specific info */

	/* initialize device ptr's */
	for (i=0; i<MAX_DEVICES; i++)
		PCIDevice_ptr[i] = &PCIDevice[i];

	/* initialize default log file name */
	strncpy(logfilename, DEF_LOGNAME, 13);
	logfilename_ptr = logfilename;

	/* get cmd line switches */
	ParseCmdLine(&flags, argc, argv);

	/* if the user wants verbose mode then all output will go to the screen */
	/* we indicate this by setting logfilename_ptr = NULL */
	if (flags.verbose) logfilename_ptr = NULL;

	WriteLog(LOG_PASS,LOG_DETAIL,"\nSymbios Logic Validation Test Code\n",NULL);
	sprintf(string,"Part(s) under test: %s\n", DEVICE_UNDER_TEST);
	WriteLog(LOG_PASS,LOG_DETAIL,string,NULL);
	sprintf(string,"%s Test, Version %s\n", TEST_NAME, TEST_VERSION);
	WriteLog(LOG_PASS,LOG_DETAIL,string,NULL);
	sprintf(string,"Validation Core Version %s\n", CORE_VERSION);
	WriteLog(LOG_PASS,LOG_DETAIL,string,NULL);

	/* check for pci bios and print info about it */
	PCI_check_pci_bios(&pcibios);
	if (flags.verbose)
	{
		fprintf(stdout, "Press any key to continue...");
		getch();
		fprintf(stdout, "\n");
	}

	/* build the device table */
	NumPCIDevices = C8XX_BuildPCIDeviceList(&PCIDevice_ptr, MAX_DEVICES);

	if (!NumPCIDevices) /* are there any symbios devices */
	{
		WriteLog(LOG_FAIL, LOG_DETAIL,"\nNo Symbios 53C8XX devices found!!\n", NULL);
		WriteLog(LOG_FAIL, LOG_DETAIL,"Cannot proceed with tests!!\n", NULL);
		exit(1);
	}

	if (NumPCIDevices > 1)
	{
		if (flags.device == -1)
		{
			/* We only need this if the user didn't specify a device so no need */
			/* to ever write it to the log file */
			fprintf(stdout, "\n%d Symbios 53C8XX devices found\n\n", NumPCIDevices);
			SelPCIDevice = C8XX_ChoosePCIDevice(&PCIDevice_ptr, NumPCIDevices);
		}
		else
			SelPCIDevice = flags.device;
	}
	else
		SelPCIDevice = 0;

	/* Check for a known device type */
	if (C8XX_GetChipType(PCIDevice_ptr[SelPCIDevice], &C8XXDevice) != C8XX_GOOD)
	{
		WriteLog(LOG_FAIL, LOG_DETAIL, "\nError: Unknown Device Type\n", NULL);
		WriteLog(LOG_FAIL, LOG_DETAIL, "Cannot proceed with tests!!\n", NULL);
		exit(1);
	}

	/* Get the IO Base address of the part */
	if (C8XX_GetIOBase(PCIDevice_ptr[SelPCIDevice]) != C8XX_GOOD)
	{
		WriteLog(LOG_FAIL, LOG_DETAIL, "\nError: Chosen device was not mapped to IO space!!\n", NULL);
		WriteLog(LOG_FAIL, LOG_DETAIL, "Cannot proceed with tests!!\n", NULL);
		exit(1);
	}

	/* Check for script RAM */
	if (C8XXDevice.features & C8XX_SRAM)
	{
		if (C8XX_GetRAMBase(PCIDevice_ptr[SelPCIDevice]) != C8XX_GOOD)
		{
			WriteLog(LOG_FAIL, LOG_DETAIL, "\nWarning: Chosen device's Script RAM was not mapped!!\n", NULL);
			if (flags.verbose)
			{
				fprintf(stdout, "Press any key to continue...");
				getch();
				fprintf(stdout, "\n");
			}
		}
	}

	/* List important PCI config registers */
	PCI_print_config(PCIDevice_ptr[SelPCIDevice]);
	if (flags.verbose)
	{
		fprintf(stdout, "Press any key to continue...");
		getch();
		fprintf(stdout, "\n");
	}

	/* initialize the chip to a default functional state */
	init_8XX(PCIDevice_ptr[SelPCIDevice]);

	/* initialize the all general purpose scripts */
	if (init_gpscripts() != SCSI_GOOD)
	{
		WriteLog(LOG_FAIL, LOG_DETAIL, "Error: Unable to allocate memory for "
							"general purpose scripts\n",NULL);
		WriteLog(LOG_FAIL, LOG_DETAIL, "Cannot proceed with tests!!\n", NULL);
		exit(1);
	}

	if (flags.verbose)   /* Do this only if the -v option was sel */
	{
		fprintf(stdout, "\nPress any key to start %s test...", TEST_NAME);
		getch();
		fprintf(stdout, "\n");
	}

	if (!test(PCIDevice_ptr[SelPCIDevice], &flags))
	{
		sprintf(string, "Test %s failed.\n", TEST_NAME);
		WriteLog(LOG_FAIL, LOG_DETAIL, string, NULL);
	}
	else
	{
		sprintf(string, "Test %s passed.\n", TEST_NAME);
		WriteLog(LOG_PASS, LOG_DETAIL, string, NULL);
	}

	if (flags.verbose)    /* Do this only if the -v option was sel */
	{
		fprintf(stdout, "\nPress any key to exit %s test...", TEST_NAME);
		getch();
	}

}


